import { getContext, setContext } from 'svelte';
import { LocalizationService } from './service.svelte.js';
const DefaultActiveLocaleSearchOptions = {
    params: ['lang', 'locale', 'language'],
    searchParams: ['lang', 'locale', 'language'],
    cookies: ['lang', 'locale', 'language']
};
class LocalizationFactoryImpl {
    __instance = undefined;
    __config = undefined;
    __commonServiceConfig = undefined;
    configure = (config) => {
        this.__config = config;
    };
    get config() {
        if (!this.__config) {
            throw new Error('Localization Service Factory not initialized, use configure() first');
        }
        return this.__config;
    }
    setCommonServiceConfig = (config) => {
        if (config.availableLocales.length === 0) {
            throw new Error('At least one available locale must be provided');
        }
        this.__commonServiceConfig = config;
    };
    get commonServiceConfig() {
        if (!this.__commonServiceConfig) {
            throw new Error('Localization Common Service Config not initialized, use setCommonServiceConfig() first');
        }
        return this.__commonServiceConfig;
    }
    importLoaderFactory = () => {
        return importLoaderFactory(this.config.importDirPath, this.config.importLoads);
    };
    extractLocales = (event, searchOptions = DefaultActiveLocaleSearchOptions) => {
        return event.untrack(() => {
            const availableLocales = this.commonServiceConfig.availableLocales;
            // Extract requested locales from headers or from navigator
            const requestedLocales = this.__config?.browser
                ? [...navigator.languages]
                : (event.request.headers
                    .get('accept-language')
                    ?.split(',')
                    .map((locale) => locale.split(';', 1)[0]) ?? []);
            // Now try to finde the active locale
            let activeLocale = undefined;
            // Check event params
            if (searchOptions.params) {
                for (const param of searchOptions.params) {
                    if (event.params[param]) {
                        activeLocale = event.params[param];
                        break;
                    }
                }
            }
            // Check event cookies
            if (!activeLocale && searchOptions.cookies) {
                for (const param of searchOptions.cookies) {
                    activeLocale = event.cookies.get(param);
                    if (activeLocale) {
                        break;
                    }
                }
            }
            // Check event query
            if (!activeLocale && searchOptions.searchParams) {
                for (const param of searchOptions.searchParams) {
                    activeLocale = event.url.searchParams.get(param);
                    if (activeLocale) {
                        break;
                    }
                }
            }
            // If nothing was found use the first available locale
            if (!activeLocale) {
                activeLocale =
                    requestedLocales.find((locale) => availableLocales.includes(locale)) ||
                        availableLocales[0];
            }
            return {
                availableLocales,
                requestedLocales,
                activeLocale
            };
        });
    };
    getContextService = () => {
        if (!this.config.browser) {
            const service = getContext(this.config.contextName)?.service();
            if (!service) {
                throw new Error(`Localization service not found in context ${this.config.contextName}`);
            }
            return service;
        }
        if (!this.__instance) {
            throw new Error('Localization service is not initialized, use initialLoadLocalizations() first');
        }
        return this.__instance;
    };
    setContextService = (service) => {
        setContext(this.config.contextName, { service: () => service });
    };
    initialLoadLocalizations = async (pathname, config) => {
        const _config = { ...this.commonServiceConfig, ...config };
        if (!_config.activeLocale) {
            throw new Error('activeLocale is required in initialLoadLocalizations');
        }
        const service = this.createService(_config);
        await service.loadLocalizations(pathname);
        return service;
    };
    createService = (config) => {
        if (!this.config.browser) {
            return new LocalizationService(config);
        }
        if (!this.__instance) {
            this.__instance = new LocalizationService(config);
        }
        return this.__instance;
    };
}
function importLoaderFactory(importDirPath, importLoads) {
    return (importFilePath) => {
        return async (locale) => {
            const importLocalizationPath = `${importDirPath}/${locale}/${importFilePath}`;
            const importFunc = importLoads[importLocalizationPath];
            if (!importFunc)
                return undefined;
            const data = (await importFunc());
            return data.default;
        };
    };
}
export const LocalizationFactory = new LocalizationFactoryImpl();
